@@ -5,40 +5,14 @@ from selenium.webdriver.common.by import By
 
 class setup_shortcuts:
     '''Shortcut functions for Selenium'''
-    def __init__(self, browser='chrome', port=9222, wait_for=10, executable_path='chromedriver', headless=False, use_uc=False, browser_version=None, driver=None):
+    def __init__(self, browser='chrome', port=9222, wait_for=10, executable_path='chromedriver', headless=False, use_uc=False, browser_version=None, driver=None, profile=None, additional_options=[]):
         self.browser_process_id = None
 
         if not driver:
             try: # Check if Chrome is already started
-                response = requests.get(f'http://localhost:{port}/json')
+                requests.get(f'http://localhost:{port}/json')
             except requests.exceptions.ConnectionError: # Otherwise, start Chrome
-                possible_chrome_locations = []
-                if 'darwin' in sys.platform:
-                    if browser == 'chromium':
-                        possible_chrome_locations.append('/Applications/Chromium.app/Contents/MacOS/Chromium')
-                    else:
-                        possible_chrome_locations.append('/Applications/Google Chrome.app/Contents/MacOS/Google Chrome')
-                else:
-                    system_drive = os.environ.get('SYSTEMDRIVE') # 'C:\'
-                    folders_chrome_could_be_installed_to = [
-                        os.environ.get('PROGRAMFILES'),
-                        os.environ.get('PROGRAMFILES(X86)'),
-                        os.environ.get('LOCALAPPDATA')
-                        ]
-                    for folder in folders_chrome_could_be_installed_to:
-                        if browser == 'chromium':
-                            possible_chrome_locations.append(f'{folder}/Chromium/Application/chrome.exe')
-                        else:
-                            possible_chrome_locations.append(f'{folder}/Google/Chrome/Application/chrome.exe')
-                            
-                for candidate in possible_chrome_locations:
-                    if os.path.exists(candidate):
-                        launch_arguments = [candidate, f'--remote-debugging-port={port}', f'--user-data-dir={os.getcwd()}/profile', '--start-maximized', '--no-default-browser-check', '--no-first-run', '--credentials_enable_service=false', '--profile.password_manager_enabled=false', '--disable-save-password-bubble', '--disable-notifications']
-                        if headless:
-                            launch_arguments += ['--headless', '--window-size=1920,1080', '--start-maximized', '--no-sandbox']
-                        self.browser_process_id = subprocess.Popen(launch_arguments).pid
-                        sleep(1)
-                        break
+                self.open_chrome(browser, port, headless, profile, additional_options)
             
             options = webdriver.ChromeOptions()
             options.debugger_address = f'127.0.0.1:{port}' # Must be 127.0.0.1 for undetected-chromedriver, not localhost
@@ -141,6 +115,44 @@ class setup_shortcuts:
                     return elements
         return []
     
+    def find_chrome_location(self):
+        if sys.platform == 'win32':
+            folders_chrome_could_be_installed_to = [
+                os.environ.get('PROGRAMFILES'),
+                os.environ.get('PROGRAMFILES(X86)'),
+                os.environ.get('LOCALAPPDATA')
+                ]
+            for folder in folders_chrome_could_be_installed_to:
+                chrome_location = f'{folder}/Google/Chrome/Application/chrome.exe'
+                if os.path.exists(chrome_location):
+                    return chrome_location
+        else:
+            chrome_location = '/Applications/Google Chrome.app/Contents/MacOS/Google Chrome'
+            if os.path.exists(chrome_location):
+                return chrome_location
+        raise ChromeStartupException("Couldn't find a Chrome install")
+    
+    def open_chrome(self, browser, port, headless, profile, additional_options):
+        chrome_location = self.find_chrome_location()
+
+        if not profile:
+            profile = f'--user-data-dir={os.getcwd()}/profile'
+        launch_arguments = [
+            f'--remote-debugging-port={port}',
+            profile,
+            '--no-default-browser-check', # Works
+            '--credentials_enable_service=false', '--profile.password_manager_enabled=false', '--disable-save-password-bubble', '--disable-notifications' # Unknown
+            ]
+        if not os.path.exists(profile) and all('window-size' not in option for option in additional_options):
+            launch_arguments.append('--start-maximized')
+        if headless:
+            launch_arguments += ['--headless', '--window-size=1920,1080', '--no-sandbox']
+        if additional_options:
+            launch_arguments += additional_options
+
+        self.browser_process_id = subprocess.Popen([chrome_location] + launch_arguments).pid
+        sleep(1)
+    
     def close(self):
         if self.browser_process_id:
             os.kill(self.browser_process_id, 15)
